from django.utils.functional import SimpleLazyObject

from .models.leprikonsite import LeprikonSite


def leprikonsite(request):
    return {'leprikonsite': SimpleLazyObject(LeprikonSite.objects.get_current)}



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-02-11 04:57
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import djangocms_text_ckeditor.fields


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0024_subjectregistration_note'),
    ]

    operations = [
        migrations.CreateModel(
            name='Agreement',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, unique=True, verbose_name='name')),
                ('order', models.IntegerField(blank=True, default=0, verbose_name='order')),
                ('heading', models.CharField(blank=True, default='', max_length=50, verbose_name='heading')),
                ('agreement', djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', verbose_name='agreement')),
            ],
            options={
                'verbose_name': 'legal agreement',
                'verbose_name_plural': 'legal agreements',
                'ordering': ('order',),
            },
        ),
        migrations.CreateModel(
            name='AgreementOption',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, verbose_name='name')),
                ('order', models.IntegerField(blank=True, default=0, verbose_name='order')),
                ('required', models.BooleanField(default=False, verbose_name='required')),
                ('option', djangocms_text_ckeditor.fields.HTMLField(verbose_name='option')),
                ('agreement', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='options', to='leprikon.Agreement', verbose_name='agreement')),
            ],
            options={
                'verbose_name': 'agreement option',
                'verbose_name_plural': 'agreement options',
                'ordering': ('order',),
            },
        ),
        migrations.RenameField(
            model_name='leprikonsite',
            old_name='registration_agreement',
            new_name='old_registration_agreement',
        ),
        migrations.RenameField(
            model_name='subject',
            old_name='agreement',
            new_name='old_registration_agreement',
        ),
        migrations.RenameField(
            model_name='subjecttype',
            old_name='agreement',
            new_name='old_registration_agreement',
        ),
        migrations.AlterField(
            model_name='leprikonsite',
            name='old_registration_agreement',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', help_text='This agreement will be removed in future version. Please, use registration agreements below.', verbose_name='old registration agreement'),
        ),
        migrations.AlterField(
            model_name='subject',
            name='old_registration_agreement',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', help_text='This agreement will be removed in future version. Please, use registration agreements below.', verbose_name='old registration agreement'),
        ),
        migrations.AlterField(
            model_name='subjecttype',
            name='old_registration_agreement',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', help_text='This agreement will be removed in future version. Please, use registration agreements below.', verbose_name='old registration agreement'),
        ),
        migrations.AddField(
            model_name='leprikonsite',
            name='registration_agreements',
            field=models.ManyToManyField(blank=True, help_text='Add legal agreements for the registration form.', related_name='_leprikonsite_registration_agreements_+', to='leprikon.Agreement', verbose_name='registration agreements'),
        ),
        migrations.AddField(
            model_name='subject',
            name='registration_agreements',
            field=models.ManyToManyField(blank=True, help_text='Add additional legal agreements for the registration form.', related_name='_subject_registration_agreements_+', to='leprikon.Agreement', verbose_name='additional registration agreements'),
        ),
        migrations.AddField(
            model_name='subjectregistration',
            name='agreement_options',
            field=models.ManyToManyField(blank=True, to='leprikon.AgreementOption'),
        ),
        migrations.AddField(
            model_name='subjecttype',
            name='registration_agreements',
            field=models.ManyToManyField(blank=True, help_text='Add additional legal agreements for the registration form.', related_name='_subjecttype_registration_agreements_+', to='leprikon.Agreement', verbose_name='additional registration agreements'),
        ),
        migrations.AlterUniqueTogether(
            name='agreementoption',
            unique_together=set([('agreement', 'name')]),
        ),
    ]



import locale

import icu
from django.contrib import admin
from django.contrib.auth import get_user_model
from django.http import Http404
from django.template.response import TemplateResponse
from django.utils.translation import ugettext_lazy as _

from ..models.account import AccountClosure
from ..models.agegroup import AgeGroup
from ..models.agreements import Agreement
from ..models.citizenship import Citizenship
from ..models.courses import Course, CourseDiscount, CourseRegistration
from ..models.department import Department
from ..models.events import Event, EventDiscount, EventRegistration
from ..models.journals import JournalEntry, JournalLeaderEntry
from ..models.leprikonsite import LeprikonSite
from ..models.messages import Message, MessageRecipient
from ..models.place import Place
from ..models.printsetup import PrintSetup
from ..models.question import Question
from ..models.roles import Leader, Parent, Participant
from ..models.school import School
from ..models.schoolyear import SchoolYear, SchoolYearDivision
from ..models.subjects import (
    Subject, SubjectGroup, SubjectPayment, SubjectRegistration, SubjectType,
)
from ..models.timesheets import Timesheet, TimesheetEntry, TimesheetEntryType
from .account import AccountClosureAdmin
from .agegroup import AgeGroupAdmin
from .agreements import AgreementAdmin
from .citizenship import CitizenshipAdmin
from .courses import CourseAdmin, CourseDiscountAdmin, CourseRegistrationAdmin
from .department import DepartmentAdmin
from .events import EventAdmin, EventDiscountAdmin, EventRegistrationAdmin
from .journals import JournalEntryAdmin, JournalLeaderEntryAdmin
from .leprikonsite import LeprikonSiteAdmin
from .messages import MessageAdmin, MessageRecipientAdmin
from .place import PlaceAdmin
from .printsetup import PrintSetupAdmin
from .question import QuestionAdmin
from .roles import LeaderAdmin, ParentAdmin, ParticipantAdmin
from .school import SchoolAdmin
from .schoolyear import SchoolYearAdmin, SchoolYearDivisionAdmin
from .subjects import (
    SubjectAdmin, SubjectGroupAdmin, SubjectPaymentAdmin,
    SubjectRegistrationAdmin, SubjectTypeAdmin,
)
from .timesheets import (
    TimesheetAdmin, TimesheetEntryAdmin, TimesheetEntryTypeAdmin,
)
from .user import UserAdmin

User = get_user_model()

admin.site.site_header = _('Leprikon administration')
admin.site.site_title = _('Leprikon administration')

admin.site.register(AccountClosure,             AccountClosureAdmin)

admin.site.register(AgeGroup,                   AgeGroupAdmin)
admin.site.register(Agreement,                  AgreementAdmin)
admin.site.register(Citizenship,                CitizenshipAdmin)
admin.site.register(LeprikonSite,               LeprikonSiteAdmin)
admin.site.register(Place,                      PlaceAdmin)
admin.site.register(PrintSetup,                 PrintSetupAdmin)
admin.site.register(School,                     SchoolAdmin)
admin.site.register(SchoolYear,                 SchoolYearAdmin)
admin.site.register(SchoolYearDivision,         SchoolYearDivisionAdmin)

admin.site.register(Leader,                     LeaderAdmin)
admin.site.register(Parent,                     ParentAdmin)
admin.site.register(Participant,                ParticipantAdmin)

admin.site.register(Subject,                    SubjectAdmin)
admin.site.register(SubjectType,                SubjectTypeAdmin)
admin.site.register(SubjectGroup,               SubjectGroupAdmin)
admin.site.register(SubjectRegistration,        SubjectRegistrationAdmin)
admin.site.register(SubjectPayment,             SubjectPaymentAdmin)

admin.site.register(Course,                     CourseAdmin)
admin.site.register(CourseRegistration,         CourseRegistrationAdmin)
admin.site.register(CourseDiscount,             CourseDiscountAdmin)
admin.site.register(JournalEntry,         JournalEntryAdmin)
admin.site.register(JournalLeaderEntry,   JournalLeaderEntryAdmin)

admin.site.register(Department,                 DepartmentAdmin)

admin.site.register(Event,                      EventAdmin)
admin.site.register(EventRegistration,          EventRegistrationAdmin)
admin.site.register(EventDiscount,              EventDiscountAdmin)

admin.site.register(Message,                    MessageAdmin)
admin.site.register(MessageRecipient,           MessageRecipientAdmin)

admin.site.register(Question,                   QuestionAdmin)

admin.site.register(Timesheet,                  TimesheetAdmin)
admin.site.register(TimesheetEntry,             TimesheetEntryAdmin)
admin.site.register(TimesheetEntryType,         TimesheetEntryTypeAdmin)

admin.site.unregister(User)
admin.site.register(User,                       UserAdmin)


def app_index(self, request, app_label, extra_context=None):
    app_dict = self._build_app_dict(request, app_label)
    if not app_dict:
        raise Http404('The requested admin page does not exist.')
    # Sort the models alphabetically within each app.
    collator = icu.Collator.createInstance(icu.Locale('.'.join(locale.getlocale())))
    app_dict['models'].sort(key=lambda x: collator.getSortKey(x['name'].lower()))
    context = dict(
        self.each_context(request),
        title=_('Leprikon administration'),
        app_list=[app_dict],
        app_label=app_label,
    )
    context.update(extra_context or {})

    request.current_app = self.name

    return TemplateResponse(request, self.app_index_template or [
        'admin/%s/app_index.html' % app_label,
        'admin/app_index.html'
    ], context)


def get_app_list(self, request):
    """
    Returns a sorted list of all the installed apps that have been
    registered in this site.
    """
    collator = icu.Collator.createInstance(icu.Locale('.'.join(locale.getlocale())))

    app_dict = self._build_app_dict(request)

    # Sort the apps alphabetically.
    app_list = sorted(app_dict.values(), key=lambda x: collator.getSortKey(x['name'].lower()))

    # Sort the models alphabetically within each app.
    for app in app_list:
        app['models'].sort(key=lambda x: collator.getSortKey(x['name'].lower()))

    return app_list


# override default Admin site's app_index and get_app_list
admin.sites.AdminSite.app_index = app_index
admin.sites.AdminSite.get_app_list = get_app_list



from datetime import datetime, timedelta

from django.core.urlresolvers import reverse_lazy as reverse
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from djangocms_text_ckeditor.fields import HTMLField

from .startend import StartEndMixin
from .subjects import Subject, SubjectRegistration


def get_default_agenda():
    return '<p>{}</p>'.format(_('instruction on OSH'))


@python_2_unicode_compatible
class JournalEntry(StartEndMixin, models.Model):
    subject     = models.ForeignKey(Subject, verbose_name=_('subject'), editable=False,
                                    related_name='journal_entries', on_delete=models.PROTECT)
    date        = models.DateField(_('date'))
    start       = models.TimeField(_('start time'), blank=True, null=True)
    end         = models.TimeField(_('end time'), blank=True, null=True)
    agenda      = HTMLField(_('session agenda'), default=get_default_agenda)
    registrations = models.ManyToManyField(SubjectRegistration, verbose_name=_('participants'), blank=True,
                                           related_name='journal_entries')

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('date', 'start', 'end')
        verbose_name        = _('journal entry')
        verbose_name_plural = _('journal entries')

    def __str__(self):
        return '{subject_name}, {date}, {duration}'.format(
            subject_name=self.subject.name,
            date=self.date,
            duration=self.duration,
        )

    @cached_property
    def datetime_start(self):
        try:
            return datetime.combine(self.date, self.start)
        except:
            return None

    @cached_property
    def datetime_end(self):
        try:
            return datetime.combine(self.date, self.end)
        except:
            return None

    @cached_property
    def duration(self):
        try:
            return self.datetime_end - self.datetime_start
        except:
            return timedelta()
    duration.short_description = _('duration')

    @cached_property
    def all_registrations(self):
        return list(self.registrations.all())

    @cached_property
    def all_registrations_idset(self):
        return set(r.id for r in self.all_registrations)

    @cached_property
    def all_leader_entries(self):
        return list(self.leader_entries.all())

    @cached_property
    def all_leader_entries_by_leader(self):
        return dict((e.timesheet.leader, e) for e in self.all_leader_entries)

    @cached_property
    def all_leaders(self):
        return list(
            le.timesheet.leader for le in self.all_leader_entries
            if le.timesheet.leader in self.subject.all_leaders
        )

    @cached_property
    def all_alternates(self):
        return list(
            le.timesheet.leader for le in self.all_leader_entries
            if le.timesheet.leader not in self.subject.all_leaders
        )

    @property
    def timesheets(self):
        from .timesheets import Timesheet
        return Timesheet.objects.by_date(self.start).filter(
            leader__in = self.all_leaders + self.all_alternates,
        )

    def save(self, *args, **kwargs):
        if self.end is None:
            self.end = self.start
        super(JournalEntry, self).save(*args, **kwargs)

    def get_edit_url(self):
        return reverse('leprikon:journalentry_update', args=(self.id,))

    def get_delete_url(self):
        return reverse('leprikon:journalentry_delete', args=(self.id,))


@python_2_unicode_compatible
class JournalLeaderEntry(StartEndMixin, models.Model):
    journal_entry = models.ForeignKey(JournalEntry, verbose_name=_('journal entry'),
                                      related_name='leader_entries', editable=False)
    timesheet   = models.ForeignKey('leprikon.Timesheet', verbose_name=_('timesheet'), related_name='journal_entries',
                                    editable=False, on_delete=models.PROTECT)
    start       = models.TimeField(_('start time'))
    end         = models.TimeField(_('end time'))

    class Meta:
        app_label           = 'leprikon'
        verbose_name        = _('journal leader entry')
        verbose_name_plural = _('journal leader entries')
        unique_together     = (('journal_entry', 'timesheet'),)

    def __str__(self):
        return '{subject_name}, {date}, {duration}'.format(
            subject_name=self.journal_entry.subject.name,
            date=self.journal_entry.date,
            duration=self.duration,
        )

    @cached_property
    def date(self):
        return self.journal_entry.date
    date.short_description = _('date')
    date.admin_order_field = 'journal_entry__date'

    @cached_property
    def subject(self):
        return self.journal_entry.subject
    subject.short_description = _('subject')

    @cached_property
    def datetime_start(self):
        return datetime.combine(self.date, self.start)

    @cached_property
    def datetime_end(self):
        return datetime.combine(self.date, self.end)

    @cached_property
    def duration(self):
        return self.datetime_end - self.datetime_start
    duration.short_description = _('duration')

    @property
    def group(self):
        return self.subject

    def get_edit_url(self):
        return reverse('leprikon:journalleaderentry_update', args=(self.id,))

    def get_delete_url(self):
        return reverse('leprikon:journalleaderentry_delete', args=(self.id,))



from datetime import date
from datetime import timedelta

from django.contrib.auth.models import User

from leprikon.models.schoolyear import SchoolYear
from leprikon.models.subjects import SubjectType

school_year = SchoolYear.objects.get_current()
current_period = school_year.periods.get_or_create(
    name='current period',
    start=date.today() - timedelta(10),
    end=date.today() + timedelta(10),
)[0]
next_period = school_year.periods.get_or_create(
    name='next period',
    start=date.today() + timedelta(11),
    end=date.today() + timedelta(22),
)[0]

subject_type_event = SubjectType.objects.get_or_create(subject_type=SubjectType.EVENT, name='event')
subject_type_course = SubjectType.objects.get_or_create(subject_type=SubjectType.COURSE, name='course')

user1 = User.objects.get_or_create(username='testuser1')[0]
